// ==========================================
//              Launch EDITOR
// ==========================================

function disableEnterInForm() {
    // Disable 'submit' action by Enter inside a form field
    $('form main input').on('keypress', function (e) { if(e.which == 13){ return false; }; });
};

// ==========================================
//             Disabled & Hidden
// ==========================================

function initializeDisabled() {
    $("form section input.disabled, form section textarea.disabled, form section table.disabled").each(function() {
        $section = $(this).closest("section");
        $section.find("input, label, textarea, table td").css({ "opacity": "0.5" }).unbind().focus(function() {
            infoPopup(LOCALIZE["disabled-input-alert"]);
            $(this).blur();
            return false;
        });
        $section.find(".media .delete, .media .sort, .media .upload, ul.checkbox, ul.radio").hide();
    });
    $("form section .hide_label").each(function() {
        $(this).closest("section").find("p.label, p.description").hide();
    });
};

// ==========================================
//                   Table
// ==========================================

function tableHeadings() {
    $("div.table table").each(function() {
        n = -1;
        $(this).find("tr").first().find("td").each(function() {
            if(n > -1) { $(this).text( String.fromCharCode(65 + n) ); };
            n++;
        });
        n = 0;
        $(this).find("tr").each(function() {
            if(n > 0) { $(this).children("td").first().text(n); };
            n++;
        });
    });
};

function initializeTable() {
    // Add table row extra button
    $(".add_table_row").unbind("click").click(function() {
        $row = $(this).closest("section").find("table." + LANG + " tr").last();
        if($row.length) {
            num = $row.attr("class").split("_").pop();
            $row.after( $row.clone() );
            $row.next().find("td").text("");
            $row.next().attr("class", "num_" + ++num);
            tableHeadings();
            initializeTable();
            initializeDectectChanges();
        };
        $(this).blur();
        return false;
    });

    // Edit row
    $("table .edit_row").unbind("contextmenu").bind("contextmenu", function() {
        // ====== Variables ======
        $table = $(this).closest("table");
        row_id = $(this).closest("tr").attr("class").split(" ").shift();
        // ====== Row Highlight ======
        $("tr").css({ "background-color": "inherit" });
        $(this).closest("tr").css({ "background-color": "#ccffee" });
        // ====== Menu Popup ======
        html = "<ul id='table_edit'>" +
            "<li class='move_up'>" + LOCALIZE["move-row-up"] + "</li>" +
            "<li class='move_down'>" + LOCALIZE["move-row-down"] + "</li>" +
            "<li class='new_above'>" + LOCALIZE["add-row-above"] + "</li>" +
            "<li class='new_below'>" + LOCALIZE["add-row-below"] + "</li>" +
            "<li class='delete_row'>" + LOCALIZE["delete-row"] + "</li>" +
            "</ul>";
        xy = $(this).offset();
        x = xy.left + $(this).width() + (parseInt( $(this).css("padding-left") ) * 2);
        y = xy.top - $(window).scrollTop() + $(this).height() + (parseInt( $(this).css("padding-top") ) * 2);
        $("body").append(html);
        $("#table_edit").fadeIn(250).css({ "bottom": $(window).height() - y, "right": $(window).width() - x, "min-width": "auto" });
        // ====== Menu Actions ======
        // Hide on mouseleave
        $("#table_edit").unbind("mouseleave").mouseleave(function() {
            $(this).fadeOut(250, function() { $(this).remove(); });
            $("tr").css({ "background-color": "inherit" });
        });
        // Hide after click
        $("#table_edit li").unbind("click").click(function() {
            $("#table_edit").fadeOut(250, function() { $(this).remove(); });
            $("tr").css({ "background-color": "inherit" });
        });
        // Delete Row
        $("#table_edit .delete_row").click(function() {
            if($table.find("tr").length > 2) { // preserve the last data column!
                $table.find("tr." + row_id).fadeOut(250, function() {
                    $(this).remove();
                    tableHeadings();
                });
            }
            else {
                infoPopup(LOCALIZE["only-row-alert"]);
            };
        });
        // Move Row
        $("#table_edit .move_up, #table_edit .move_down").click(function() {
            $row = $table.find("tr." + row_id);
            if($(this).attr("class") == "move_up") {
                if( $row.prev().prev().length ) {
                    $row.fadeOut(250, function() {
                        $row.prev().before( $row.clone() );
                        $row.prev().prev().fadeIn(250);
                        $row.remove();
                    });
                }
                else {
                    infoPopup(LOCALIZE["cant-go-further"]);
                };
            }
            else {
                if( $row.next().length ) {
                    $row.fadeOut(250, function() {
                        $row.next().after( $row.clone() );
                        $row.next().next().fadeIn(250);
                        $row.remove();
                    });
                }
                else {
                    infoPopup(LOCALIZE["cant-go-further"]);
                };
            };
            setTimeout(function() {
                tableHeadings();
                initializeTable();
                initializeDectectChanges();
            }, 500);
        });
        // New Row
        $("#table_edit .new_above, #table_edit .new_below").click(function() {
            // find new tr (class) id
            new_id = 0;
            $table.find("tr").each(function() {

                id = $(this).attr("class").split("_");
                if(id.length == 2 && parseInt(id[1]) >= new_id) { new_id = parseInt(id[1]) + 1; };

                //alert("id: " + $(this).attr("class") + " / " + new_id);
            });

            $row = $table.find("tr." + row_id);

            if($(this).attr("class") == "new_below") {
                $row.after( $row.clone() );
                $row.next().find("td").text("");
                $row.next().attr("class", "num_" + new_id);
            }
            else {
                $row.before( $row.clone() );
                $row.prev().find("td").text("");
                $row.prev().attr("class", "num_" + new_id);
            };
            tableHeadings();
            initializeTable();
            initializeDectectChanges();
        });
        return false; // block stock context menu
    });

    // Edit column
    $("table .edit_column").unbind("click").bind("contextmenu", function() {
        // ====== Variables ======
        $table = $(this).closest("table");
        col_id = $(this).attr("class").split(" ").shift();
        // ====== Column Highlight ======
        resetTableColumnColor();
        $table.find("td." + col_id).each(function() {
            if($(this).closest("tr").attr("class") != "start") {
                $(this).css({ "background-color": "#ccffee" });
            };
        });
        // ====== Menu Popup ======
        $("#table_edit").remove();
        html = "<ul id='table_edit'>" +
            "<li class='move_left'>" + LOCALIZE["move-column-left"] + "</li>" +
            "<li class='move_right'>" + LOCALIZE["move-column-right"] + "</li>" +
            "<li class='new_left'>" + LOCALIZE["add-column-left"] + "</li>" +
            "<li class='new_right'>" + LOCALIZE["add-column-right"] + "</li>" +
            "<li class='delete_column'>" + LOCALIZE["delete-column"] + "</li>" +
            "</ul>";
        xy = $(this).offset();
        x = xy.left + $(this).width() + (parseInt( $(this).css("padding-left") ) * 2);
        y = xy.top - $(window).scrollTop() + $(this).height() + (parseInt( $(this).css("padding-top") ) * 2);
        wid = $(this).width() + (parseInt( $(this).css("padding-left") ) * 2);
        $("body").append(html);
        $("#table_edit").fadeIn(250).css({ "bottom": $(window).height() - y, "right": $(window).width() - x, "min-width": wid });

        // ====== Menu Actions ======
        // Hide on mouseleave
        $("#table_edit").unbind("mouseleave").mouseleave(function() {
            $("#table_edit").fadeOut(250, function() { $(this).remove(); });
            resetTableColumnColor();
        });
        // Hide after click
        $("#table_edit li").unbind("click").click(function() {
            $("#table_edit").fadeOut(250, function() { $(this).remove(); });
            resetTableColumnColor();
        });
        // Delete Column
        $("#table_edit .delete_column").click(function() {
            if($table.find("tr").first().find("td").length > 2) { // preserve the last data column!
                $table.find("td." + col_id).fadeOut(250, function() {
                    $(this).remove();
                    tableHeadings();
                });
            }
            else {
                infoPopup(LOCALIZE["only-column-alert"]);
            };
        });
        // Move Column
        $("#table_edit .new_right, #table_edit .new_left").click(function() {
            $column = $table.find("td." + col_id);
            dir = $(this).attr("class");
            // Find new id
            new_id = 0;
            $table.find("tr").first().find("td").each(function() {
                id = $(this).attr("class").split("_");
                if(id[0] == "num" && id[1].charCodeAt(0) >= new_id) { new_id = id[1].charCodeAt(0) + 1; };
            });
            new_id = "num_" + String.fromCharCode(new_id);
            // Move column
            $table.find("td." + col_id).each(function() {
                if(dir == "new_left") {
                    $(this).before( $(this).clone() );
                    $new = $(this).prev();
                }
                else {
                    $(this).after( $(this).clone() );
                    $new = $(this).next();
                };
                $new.removeClass(col_id);
                $new.attr("class", new_id + " " + $new.attr("class"));
                $new.text("");
                tableHeadings();
                initializeTable();
                initializeDectectChanges();
            });
        });
        // New Column
        $("#table_edit .move_right, #table_edit .move_left").click(function() {
            $column = $table.find("td." + col_id);
            if($(this).attr("class") == "move_right") {
                if($column.first().next().length) {
                    $column.each(function() {
                        $(this).next().after( $(this).clone() );
                        $(this).remove();
                    }); 
                }
                else {
                    infoPopup(LOCALIZE["cant-go-further"]);
                };
            }
            else {
                if($column.first().prev().prev().length) {
                    $column.each(function() {
                        $(this).prev().before( $(this).clone() );
                        $(this).remove();
                    }); 
                }
                else {
                    infoPopup(LOCALIZE["cant-go-further"]);
                };
            };
            setTimeout(function() {
                tableHeadings();
                initializeTable();
                initializeDectectChanges();
            }, 500);
        });
        return false; // block stock context menu
    });
};

function resetTableColumnColor() {
    $("table").find("tr").each(function() {
        if($(this).attr("class") != "start") {
            $(this).children("td").each(function() {
                if($(this).attr("class").split(" ").shift() != "heading") {
                    $(this).css({ "background-color": "inherit" });
                };
            });
        };
    });
};

// ==========================================
//             Text Style Editor
// ==========================================

var KEEP_STYLE_BUTTONS = false; // Keep style buttons (after style edit)

function initializeShowStyle() {
// --------------------------------
// Show style buttons on focus
// --------------------------------
    $("textarea").focus(function() {
        $("textarea").unbind("focus");

        if($(this).prev().attr("id") == "style_buttons") {
            initializeHideStyle();
        }
        else if($("#style_buttons").length) {
            $("#style_buttons").remove();
            showStyleButtons($(this));
            initializeHideStyle();
        }
        else {
            showStyleButtons($(this));
            initializeHideStyle();
        };
        initializeHelp(); // For new style buttons
    });
};


function initializeHideStyle() {
// --------------------------------
// Hide style buttons on blur / focus change
// --------------------------------
    $("textarea").blur(function() {
        $("textarea").unbind("blur");
        initializeShowStyle();
    });
};

function showStyleButtons($textarea) {
    // ====== Show style buttons bar ======
    format = $textarea.closest("section").find("input.format").val();
    if(format != undefined && format != "") {
        html = generateStyleButtons(format);
        $textarea.before(html);
        $("#style_buttons").show(200);
    };
    // ====== Manual ======
    $("#style_buttons p").attr("help", LOCALIZE["bbcode-style-html"]);
    // ====== Long click fix ======
    $("#style_buttons span").mousedown(function() {
        KEEP_STYLE_BUTTONS = true;
    });
    // ====== Click style button ======
    $("#style_buttons span").unbind("click").click(function() {
        KEEP_STYLE_BUTTONS = true;
        applyStyle($textarea, $(this));
    });
};

function generateStyleButtons(format) {
// --------------------------------
// format = <string> buttons codes, divided by coma (eg. "b,i"):
//     "*": all buttons
//     "b": bold, "i": italic, "u": underline, "-": bullet, "l": external link, "s": in-site link, "m": mailto
// --------------------------------
// RETURNS: <string> Style buttons bar html according to format option(s)
// --------------------------------
    // Check for formatting limitations
    if($("input#site_format").length) { valid = $("input#site_format").val().split(","); }
    else { valid = false; };
    // Check for libraries
    disabled = [];
    if($("#libraries_data .files_lib").length == 0) { disabled[disabled.length] = "a"; };
    if($("#libraries_data .images_lib").length == 0) { disabled[disabled.length] = "f"; };
    if($("#libraries_data .temp_lib").length == 0) { disabled[disabled.length] = "p"; };

    if(format != "") {
        html = "";
        format = format.split(",");
        buttons = {
            "b": "<span class='fi-bold help style_bold' help='" + LOCALIZE["style-bold"] + "'></span>",
            "i": "<span class='fi-italic help style_italic' help='" + LOCALIZE["style-italic"] + "'></span>",
            "u": "<span class='fi-underline help style_underline' help='" + LOCALIZE["style-underline"] + "'></span>",
            "^": "<span class='help index style_superscript' help='" + LOCALIZE["style-superscript"] + "'>x<sup>2</sup></span>",
            "v": "<span class='help index style_subscript' help='" + LOCALIZE["style-subscript"] + "'>x<sub>2</sub></span>",
            "-": "<span class='fi-list-bullet help style_list' help='" + LOCALIZE["style-list"] + "'></span>",
            "c": "<span class='fi-align-center help style_center' help='" + LOCALIZE["style-center"] + "'></span>",
            ".": "<span class='help style_bullet' help='" + LOCALIZE["style-bullet"] + "'>&bull;</span>",
            "t": "<span class='fi-arrow-right help style_tab' help='" + LOCALIZE["style-tab"] + "'></span>",
            "_": "<span class='fi-minus help style_line' help='" + LOCALIZE["style-line"] + "'></span>",
            "m": "<span class='fi-mail help style_mail' help='" + LOCALIZE["style-mail"] + "'></span>",
            "l": "<span class='fi-link help style_link' help='" + LOCALIZE["style-link"] + "'></span>",
            "a": "<span class='fi-page-filled help style_attachment' help='" + LOCALIZE["style-library-attachment"] + "'></span>",
            "f": "<span class='fi-photo help style_figure' help='" + LOCALIZE["style-library-figure"] + "'></span>",
            "p": "<span class='fi-layout help style_page' help='" + LOCALIZE["style-library-page"] + "'></span>",
        };
        for(key in buttons) {
            // Check for fisabled or not valid first
            if(disabled.indexOf(key) < 0 && (valid == false || valid.indexOf(key) > -1)) {
                if(format.indexOf("*") >= 0 || format.indexOf(key) >= 0) { html = html + buttons[key]; };
            };
        };

        html = html + "<span class='fi-prohibited help style_delete' help=" + LOCALIZE["style-delete"] + "'></span>" // Delete BBCode
        html = html + "<span class='help style_preview' help='" + LOCALIZE["style-preview"] + "'>" + LOCALIZE["style-preview-label"] + "</span>" // Preview
        return "<div id='style_buttons'>" + html + "<p class='fi-info manual help' help='-'></p></div>"; // Show manual
    }
    else {
        return "";
    };
};

function hideStylePreviewPopup() {
    $("#page_fader, #preview_popup").fadeOut(100, function() {
        $("#style_buttons").css({ "opacity": "1" });
        $(this).remove();
        $textarea.focus();
        initializeHideStyle();
        KEEP_STYLE_BUTTONS = false;
        $("body").unbind("keyup");
    });
}

function applyStyle($textarea, $button) {
// --------------------------------
// $textarea = <object>
// $button = <object>
// --------------------------------
// Apply button style to selected text in textarea
// --------------------------------
    $("#confirm_popup button").unbind();
    style = $button.attr("class").split("_").pop();
    simple = [ "bold", "italic", "underline" ];
    word = [ "list", "center" ];
    popup = [ "link", "site", "mail"];
    index = [ "superscript", "subscript" ];
    // ====== Selected text ======
    text = $textarea.val();
    beg = $textarea[0].selectionStart;
    end = $textarea[0].selectionEnd;
    selected = text.substr(beg, (end - beg));
    strlen = text.length;
    scroll = $textarea.scrollTop();
    win_scroll = $(window).scrollTop();
    // ====== PREVIEW ======
    if(style == "preview") {
        text = BBCode( $textarea.val(), SITE_ROOT );
        $("body").append( "<div id='page_fader'></div><div id='preview_popup'>" + LOCALIZE["style-preview-label"] + "</div>" );
        xy = $textarea.offset();
        wid = $textarea.width() + (2 * parseInt($textarea.css("padding-left"))) + (2 * parseInt($textarea.css("border-left")));
        hei = $textarea.height() + (2 * parseInt($textarea.css("padding-top"))) + (2 * parseInt($textarea.css("border-top")));
        $("#preview_popup").css({ "top": xy.top, "left": xy.left, "width": wid, "min-height": hei });
        $("#preview_popup").html( "<p>" + text + "</p>" );
        KEEP_STYLE_BUTTONS = true;
        $("#page_fader").fadeIn(200);
        $("#style_buttons").css({ "opacity": "0.5" });
        $("#page_fader, #preview_popup").click(function() {
            hideStylePreviewPopup();
        });
        $("body").keyup(function(e) {
            code = e.keyCode;
            if(code == 27) { // ESC
                hideStylePreviewPopup();
            }
        });
        $("#preview_popup a").each(function() {
            href = $(this).attr("href");
            target = $(this).attr("target");
            if(href.substr(0,7) == "mailto:") {
                href = "Email: <a>" + href.substr(7) + "</a>";
            }
            else {
                href = "Link: <a>" + href + "</a>";
            };
            $(this).attr("help", href).addClass("help manual");
        });
        $("#preview_popup a").click(function() { return false; });
        initializeHelp();
    }
    // ====== BULLET ======
    else if(style == "bullet") {
        $textarea.val(text.substr(0, beg) + "[*]" + text.substr(beg));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== TAB ======
    else if(style == "tab") {
        $textarea.val(text.substr(0, beg) + "[tab]" + text.substr(beg));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== HORIZONTAL LINE ======
    else if(style == "line") {
        $textarea.val(text.substr(0, beg) + "[hr]" + text.substr(beg));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== ATTACHMENT / FIGURE / PAGE ======
    else if(style == "figure" || style == "attachment" || style == "page") {
        if(style == "figure") {
            showLibPopup("images_lib", false); // Image link (single)
        }
        else if(style == "attachment") {
            if(selected == "") { showLibPopup("files_lib", true); } // simple file -> link (multiple)
            else { showLibPopup("files_lib", false); }; // text content as link (single)
        }
        else if(style == "page") {
            if(selected == "") { showLibPopup("temp_lib", true); } // simple page link -> link (multiple)
            else { showLibPopup("temp_lib", false); }; // text content as page link (single)
        };
        KEEP_STYLE_BUTTONS = true;

        $(".popup_box button.confirm").click(function() {
            var items = [];
            $("#libraries").find("li .selected").each(function() {
                val = $(this).parent().attr("value");
                if(jQuery.type(val) == "string") {
                    items[items.length] = val;
                };
            });

            if(items.length > 0) {
                // ====== FIGURE ======
                if(style == "figure") {
                    input = items.shift();
                    // Link
                    href = $("#libraries").find("input#href").val();
                    if(href != undefined && href != "") {
                        if($("#libraries").find("input#new_window").prop("checked") == true) {
                            link_open = "[url|e=" + href + "]";
                            link_close = "[/url]";
                        }
                        else {
                            link_open = "[url|i=" + href + "]";
                            link_close = "[/url]";
                        };
                    }
                    else {
                        link_open = "";
                        link_close = "";
                    };
                    // Text side (if any)
                    if(selected != "") {
                        dir = $("#libraries").find("input.side:checked").val();
                        if(dir == "right") {
                            tag = "banner|r"
                        }
                        else {
                            tag = "banner|l";
                        };
                    }
                    else {
                        size = $("#libraries").find("input.size:checked").val();
                        tag = "button|" + size;
                    }
                    // Apply attachment
                    $textarea.val(text.substr(0, beg) + "[" + tag + "]" + link_open + "[img=" + input + "]" + link_close + selected + "[/" + tag.split("|").shift() + "]" + text.substr(end));
                    hideLibPopup();
                }

                // ====== ATTACHMENT / PAGE ======
                else {
                    if(style == "page") { tag = "url|i"; }
                    else { tag = "url|e"; };
                    // Multiple files
                    if(selected == "") {
                        for(i in items) {
                            href = items[i];
                            href = "[" + tag + "=" + href + "]" + href.path("basename") + "[/" + tag.split("|").shift() + "]";
                            items[i] = href;
                        };
                        $textarea.val(text.substr(0, beg) + items.join("\n") + text.substr(end));
                    }
                    // Single link
                    else {
                        input = items.shift();
                        $textarea.val(text.substr(0, beg) + "[" + tag + "=" + input + "]" + selected + "[/" + tag.split("|").shift() + "]" + text.substr(end));
                    };
                    hideLibPopup();
                };

                // ====== Back to textarea ======
                if($("#style_buttons").length == 0) {
                    initializeHideStyle();
                    $textarea.focus();
                }
                else {
                    $textarea.focus();
                    initializeHideStyle();
                    KEEP_STYLE_BUTTONS = false;
                };
                cursorBack($textarea, scroll, win_scroll, strlen, end);
            }
            else {
                infoPopup(LOCALIZE["file-not-selected"]);
            };

        });

        $(".popup_box button.cancel").click(function() {
            hideLibPopup();
            $textarea.focus();
            initializeHideStyle();
            KEEP_STYLE_BUTTONS = true;
        });

        // Go back to textarea
        if($("#style_buttons").length == 0) {
            initializeHideStyle();
            $textarea.focus();
        }
        else {
            $textarea.focus();
            initializeHideStyle();
            KEEP_STYLE_BUTTONS = false;
        };
    }
    // ====== Lack of selection ======
    else if(selected == "") {
        infoPopup(LOCALIZE["text-not-selected"]);
        $textarea.focus();
    }
    // ====== WORD ======
    else if(word.indexOf(style) >= 0) {
        $textarea.val(text.substr(0, beg) + "[" + style + "]" + selected + "[/" + style + "]" + text.substr(end));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== INDEX ======
    else if(index.indexOf(style) >= 0) {
        tag = style.substr(0, 3);
        $textarea.val(text.substr(0, beg) + "[" + tag + "]" + selected + "[/" + tag + "]" + text.substr(end));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== SIMPLE ======
    else if(simple.indexOf(style) >= 0) {
        tag = style.substr(0, 1);
        $textarea.val(text.substr(0, beg) + "[" + tag + "]" + selected + "[/" + tag + "]" + text.substr(end));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    }
    // ====== POPUP ======
    else if (popup.indexOf(style) >= 0) {
        if(style == "link") {
            tag = "url"; title = LOCALIZE["style-link"] + ":";
            icon = "fi-link";
            info = LOCALIZE['link-input'];
        }
        else if(style == "site") {
            tag = "url"; title = LOCALIZE["style-link-internal"] + ":";
            icon = "fi-arrow-right";
            info = LOCALIZE['link-input'];
        }
        else if(style == "mail") {
            tag = "email"; title = LOCALIZE["style-mail"] + ":";
            icon = "fi-mail";
            info = LOCALIZE['mail-input'];
        }
        else { title = "?"; };
        showPopup(icon, title,  LOCALIZE["save-cancel-buttons"], info);
        $("textarea").unbind("blur");
        // Button - confirm
        $("#popup_box .confirm").unbind("click").click(function() {
            input = $("#popup_box input").val();
            if(input == "") {
                infoPopup(LOCALIZE["empty-input-alert"]);
                $("#popup_box input").focus();
            }
            //else if((style == "mail" && (input = validateEmail(input)) == false) || (style == "link" && (input = validateUrl(input)) == false)){
            else if(style == "mail" && (input = validateEmail(input)) == false) {
                infoPopup(LOCALIZE["valid-address-alert"]);
                $("#popup_box input").focus();
            }
            else if(style == "site") {
                if(input.substr(0, 7) == "http://") { input = input.substr(7); }
                else if(input.substr(0, 8) == "https://") { input = input.substr(8); };
                input = input.split("/").pop();
                if(input == "") { input = "index.php"; };
                $textarea.val(text.substr(0, beg) + "[" + tag + "=" + input + "]" + selected + "[/" + tag + "]" + text.substr(end));
                hidePopup();
            }
            else {
                $textarea.val(text.substr(0, beg) + "[" + tag + "=" + input + "]" + selected + "[/" + tag + "]" + text.substr(end));
                hidePopup();
            };
            if($("#style_buttons").length == 0) {
                initializeHideStyle();
                $textarea.focus();
            }
            else {
                $textarea.focus();
                initializeHideStyle();
                KEEP_STYLE_BUTTONS = false;
            };
            cursorBack($textarea, scroll, win_scroll, strlen, end);
        });      
        // Button cancel
        $("#popup_box .cancel").unbind().click(function() {
            hidePopup();
            $textarea.focus();
            initializeHideStyle();
            KEEP_STYLE_BUTTONS = false;
        });
    }
    // ====== DELETE BBCode ======
    else if(style = "delete") {
        $textarea.val(text.substr(0, beg) + noBBCode(selected) + text.substr(end));
        $textarea.focus();
        cursorBack($textarea, scroll, win_scroll, strlen, end);
    };
};

function cursorBack($textarea, scroll, win_scroll, strlen, end) {
// --------------------------------------------------
// Set correct cursor & textarea scroll position
// --------------------------------------------------
    $textarea.selectRange(end - (strlen - $textarea.val().length));
    textareaResize($textarea, TEXTAREA_MIN);
    // Restore document & text area scroll positions
    $textarea.scrollTop(scroll);
    $("html").scrollTop(win_scroll);
};

function hideLibPopup() {
    $("#libraries, #page_fader").fadeOut(250, function() { $(this).remove(); });
    $("body").css({ "overflow": "auto" });
};

function showLibPopup(lib, multi) {
    // Hide previous popups (if any)
    $("#page_fader, #popup_container, .popup_box, #libraries").remove();
    // Get library data
    $lib = $("#libraries_data > div." + lib);
    if($lib.length) { lib_html = $lib.get(0).outerHTML; } else { lib = ""; };
    // Image link content
    if(lib.indexOf("image") > -1) {
        icon = "fi-photo";
        inputs = "<p class='label'>Link</p>\n" +
            "<input id='href' type='text' class='text' placeholder='" + LOCALIZE['link-input'] + "'>\n" +
            "<label><input id='new_window' type='checkbox' class='checkbox' checked> " + LOCALIZE['new-window'] + "</label>\n";
        // Add text side options for photo with text banner
        if(selected != undefined && selected != "") {
            title = LOCALIZE['image-description-label'];
            inputs = inputs +
                "<p class='label'>" + LOCALIZE['image-description-position'] + "</p>\n" +
                "<label><input type='radio' class='side radio' name='side' value='left'> " + LOCALIZE['position-left'] + "</label>" +
                "<label><input type='radio' class='side radio' name='side' value='right' checked> " + LOCALIZE['position-right'] + "</label>";
        }
        else {
            title = LOCALIZE['image-label'];
            inputs = inputs +
                "<p class='label'>" + LOCALIZE['image-size'] + "</p>\n" +
                "<label><input type='radio' class='size radio' name='size' value='1'> " + LOCALIZE['size-small'] + "</label>" +
                "<label><input type='radio' class='size radio' name='size' value='2' checked> " + LOCALIZE['size-medium'] + "</label>" +
                "<label><input type='radio' class='size radio' name='size' value='3'> " + LOCALIZE['size-big'] + "</label>";
        };
        if(multi) { label = LOCALIZE['select-images']; } else { label = LOCALIZE['select-image']; };
    }
    // Page content
    else if(lib.indexOf("temp") > -1) {
        title = LOCALIZE['subpage-label'];
        icon = "fi-layout";
        inputs = "";
        if(multi) { label = LOCALIZE['select-pages']; } else { label = LOCALIZE['select-page']; };
    }
    // File content
    else {
        title = LOCALIZE['file-label'];
        icon = "fi-page-filled";
        inputs = "";
        if(multi) { label = LOCALIZE['select-files']; } else { label = LOCALIZE['select-file']; };
    };

    // HTML
    html = "<div id='page_fader'></div>\n";
    html = html + "<div id='libraries'><div class='scrollable'><div class='popup_box'>\n";
    html = html + "<h6><span class='" + icon + "'></span></h6>\n";
    html = html + "<h3>" + title + "</h3>\n";
    html = html + "<p class='label'>" + label + "</p>\n";
    html = html + $lib.get(0).outerHTML;
    html = html + inputs;
    html = html + "<div class='buttons two'><button class='confirm'>" + LOCALIZE['save-label'] + "</button><button class='cancel'>" + LOCALIZE['cancel-label'] + "</button></div>\n";
    html = html + "</div></div></div>";
    // Show
    $("body").append(html);
    $("#libraries, #page_fader").fadeIn(250);
    $("#libraries").find(".folder").first().find("li").show();
    $("body").css({ "overflow": "hidden" });
    initializeHelp();
    // ====== User Actions ======
    $("#libraries .folder .name").click(function() {
        num = $(this).parent().find("li").length;
        fold_time = num * 50;
        if(fold_time < 250) { fold_time = 250; };
        if(fold_time > 750) { fold_time = 750; };
        if(num > 0) {
            if($(this).parent().find("li").css("display") == "none") {

                $(this).parent().find("li").show(fold_time);
            }
            else {
                $(this).parent().find("li").hide(fold_time);
            }
        };
    });
    if(multi) {
        $("#libraries li").click(function() {
            $item = $(this).children().first();
            if($item.attr("class") == undefined || $item.attr("class").indexOf("selected") < 0) {
                $item.addClass("selected");
            }
            else {
                $item.removeClass("selected");
            };
        });
    }
    else {
        $("#libraries li").click(function() {
            $("#libraries li .selected").removeClass("selected");
            $item = $(this).children().first();
            $item.addClass("selected");
        });
    };
};
